/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.BeforeClass;
import org.junit.Test;
import org.ow2.petals.registry_overlay.configuration.PetalsRegistryOverlayCfgBuilder;
import org.ow2.petals.registry_overlay.configuration.PetalsRegistryOverlayCfgException;
import org.ow2.petals.registry_overlay.configuration.generated.Credentials;
import org.ow2.petals.registry_overlay.configuration.generated.ManagementConsole;
import org.ow2.petals.registry_overlay.configuration.generated.Member;
import org.ow2.petals.registry_overlay.configuration.generated.Members;
import org.ow2.petals.registry_overlay.configuration.generated.PetalsRegistryOverlay;

/**
 * Unit test about the default configuration
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DefaultConfigurationTest {

    private final static String PROPERTY_NAME_CONF_DIRECTORY = "conf-directory";

    private static File CONF_DIRECTORY = null;

    @BeforeClass
    public static void setup() {
        final String confDirectoryStr = System.getProperty(PROPERTY_NAME_CONF_DIRECTORY);
        assertNotNull("System property '" + PROPERTY_NAME_CONF_DIRECTORY + "' not set.", confDirectoryStr);
        CONF_DIRECTORY = new File(confDirectoryStr);
        assertTrue("The configuration directory '" + CONF_DIRECTORY.getAbsolutePath() + "' does not exist",
                CONF_DIRECTORY.exists());
        assertTrue("The configuration directory '" + CONF_DIRECTORY.getAbsolutePath() + "' is not a directory",
                CONF_DIRECTORY.isDirectory());
    }

    /**
     * <p>
     * Verify that the default cluster definition file is valid.
     * </p>
     * <p>
     * The cluster definition file is get through the system property {@value #PROPERTY_NAME_CONF_DIRECTORY}
     * </p>
     * <p>
     * Expected results: no error occurs parsing the default cluster definition file, and it contains a basic sample
     * cluster
     * </p>
     */
    @Test
    public void testDefaultCluster() throws PetalsRegistryOverlayCfgException {

        final File clusterFile = new File(CONF_DIRECTORY, "cluster.xml");
        final PetalsRegistryOverlay defaultCluster = PetalsRegistryOverlayCfgBuilder
                .createPetalsRegistryOverlayCfg(clusterFile);
        assertNotNull("Default topology null", defaultCluster);

        DefaultConfigurationTest.assertBasicSampleTopology(defaultCluster);
    }

    /**
     * Assert a basic sample cluster
     * 
     * @param cluster
     *            The cluster definition to check
     */
    public static void assertBasicSampleTopology(final PetalsRegistryOverlay cluster) {

        assertNotNull("Cluster is null", cluster);

        final Credentials credentials = cluster.getCredentials();
        assertNotNull("Credentials are null", credentials);
        assertEquals("Unexpected group name", "default-sample", credentials.getGroup());
        assertEquals("Unexpected group password", "s3cr3t", credentials.getPassword());

        final ManagementConsole mngtConsole = cluster.getManagementConsole();
        assertNull("Management console is not null", mngtConsole);

        final Members members = cluster.getMembers();
        assertNotNull("Members is null", members);
        assertNotNull("Member list null", members.getMember());
        assertEquals("Unexpected number of member", 1, members.getMember().size());
        final Member member = members.getMember().get(0);
        assertEquals("Unexpected member identifier", "sample-member-0", member.getId());
        assertEquals("Unexpected member host", "localhost", member.getValue());
        assertNull("Member port not null", member.getPort());

    }

}
